@@ -1,12 +1,16 @@
 import json
 import os
 from typing import Any, Dict, List, Literal, Optional, Tuple
+from aiconfig.default_parsers.claude import ClaudeBedrockModelParser
 
 import requests
 import yaml
 from aiconfig.callback import CallbackEvent, CallbackManager
-from aiconfig.default_parsers.anyscale_endpoint import DefaultAnyscaleEndpointParser
+from aiconfig.default_parsers.anyscale_endpoint import (
+    DefaultAnyscaleEndpointParser,
+)
 from aiconfig.default_parsers.openai import DefaultOpenAIParser
+from aiconfig.default_parsers.gemini import GeminiModelParser
 from aiconfig.default_parsers.palm import PaLMChatParser, PaLMTextParser
 from aiconfig.model_parser import InferenceOptions, ModelParser
 
@@ -14,36 +18,48 @@ from aiconfig.schema import JSONObject
 
 from .default_parsers.dalle import DalleImageGenerationParser
 from .default_parsers.hf import HuggingFaceTextGenerationParser
-from .registry import ModelParserRegistry, update_model_parser_registry_with_config_runtime
+from .registry import (
+    ModelParserRegistry,
+    update_model_parser_registry_with_config_runtime,
+)
 from .schema import AIConfig, Prompt
 from .util.config_utils import is_yaml_ext
 
-gpt_models = [
+gpt_models_main = [
+    "gpt-3.5-turbo",
     "gpt-4",
+]
+gpt_models_extra = [
+    "gpt-3.5-turbo-16k",
+    "gpt-3.5-turbo-0301",
+    "gpt-3.5-turbo-0613",
+    "gpt-3.5-turbo-16k-0613",
     "gpt-4-0314",
     "gpt-4-0613",
     "gpt-4-32k",
     "gpt-4-32k-0314",
     "gpt-4-32k-0613",
-    "gpt-3.5-turbo",
-    "gpt-3.5-turbo-16k",
-    "gpt-3.5-turbo-0301",
-    "gpt-3.5-turbo-0613",
-    "gpt-3.5-turbo-16k-0613",
 ]
-for model in gpt_models:
+for model in gpt_models_main:
     ModelParserRegistry.register_model_parser(DefaultOpenAIParser(model))
-ModelParserRegistry.register_model_parser(DefaultAnyscaleEndpointParser("AnyscaleEndpoint"))
-ModelParserRegistry.register_model_parser(PaLMChatParser())
-ModelParserRegistry.register_model_parser(PaLMTextParser())
-ModelParserRegistry.register_model_parser(HuggingFaceTextGenerationParser())
 dalle_image_generation_models = [
     "dall-e-2",
     "dall-e-3",
 ]
 for model in dalle_image_generation_models:
-    ModelParserRegistry.register_model_parser(DalleImageGenerationParser(model))
-
+    ModelParserRegistry.register_model_parser(
+        DalleImageGenerationParser(model)
+    )
+ModelParserRegistry.register_model_parser(
+    DefaultAnyscaleEndpointParser("AnyscaleEndpoint")
+)
+ModelParserRegistry.register_model_parser(GeminiModelParser("gemini-pro"), ["gemini-pro"])
+ModelParserRegistry.register_model_parser(ClaudeBedrockModelParser())
+ModelParserRegistry.register_model_parser(HuggingFaceTextGenerationParser())
+for model in gpt_models_extra:
+    ModelParserRegistry.register_model_parser(DefaultOpenAIParser(model))
+ModelParserRegistry.register_model_parser(PaLMChatParser())
+ModelParserRegistry.register_model_parser(PaLMTextParser())
 
 class AIConfigRuntime(AIConfig):
     # A mapping of model names to their respective parsers
@@ -127,14 +143,18 @@ class AIConfigRuntime(AIConfig):
             lastmileapi_token = os.environ.get("LASTMILE_API_TOKEN")
 
             if not lastmileapi_token:
-                raise ValueError("LASTMILE_API_TOKEN environment variable is not set.")
+                raise ValueError(
+                    "LASTMILE_API_TOKEN environment variable is not set."
+                )
 
             headers = {"Authorization": "Bearer " + lastmileapi_token}
             url = f"{API_ENDPOINT}/workbooks/aiconfig?id={workbook_id}"
             resp = requests.get(url, headers=headers)
 
             if resp.status_code != 200:
-                raise Exception(f"Failed to load workbook. Status code: {resp.status_code}")
+                raise Exception(
+                    f"Failed to load workbook. Status code: {resp.status_code}"
+                )
 
             data = resp.json()
 
@@ -177,11 +197,15 @@ class AIConfigRuntime(AIConfig):
 
         model_parser = ModelParserRegistry.get_model_parser(model_name)
         if not model_parser:
-            raise ValueError(f"Unable to serialize data: `{data}`\n Model Parser for model {model_name} does not exist.")
+            raise ValueError(
+                f"Unable to serialize data: `{data}`\n Model Parser for model {model_name} does not exist."
+            )
 
         prompts = await model_parser.serialize(prompt_name, data, self, params)
 
-        event = CallbackEvent("on_serialize_complete", __name__, {"result": prompts})
+        event = CallbackEvent(
+            "on_serialize_complete", __name__, {"result": prompts}
+        )
         await self.callback_manager.run_callbacks(event)
         return prompts
 
@@ -201,14 +225,20 @@ class AIConfigRuntime(AIConfig):
         Returns:
             str: The resolved prompt.
         """
-        event = CallbackEvent("on_resolve_start", __file__, {"prompt_name": prompt_name, "params": params})
+        event = CallbackEvent(
+            "on_resolve_start",
+            __file__,
+            {"prompt_name": prompt_name, "params": params},
+        )
         await self.callback_manager.run_callbacks(event)
 
         if not params:
             params = {}
 
         if prompt_name not in self.prompt_index:
-            raise IndexError(f"Prompt '{prompt_name}' not found in config, available prompts are:\n {list(self.prompt_index.keys())}")
+            raise IndexError(
+                f"Prompt '{prompt_name}' not found in config, available prompts are:\n {list(self.prompt_index.keys())}"
+            )
 
         prompt_data = self.prompt_index[prompt_name]
         model_name = self.get_model_name(prompt_data)
@@ -216,7 +246,9 @@ class AIConfigRuntime(AIConfig):
 
         response = await model_provider.deserialize(prompt_data, self, params)
 
-        event = CallbackEvent("on_resolve_complete", __name__, {"result": response})
+        event = CallbackEvent(
+            "on_resolve_complete", __name__, {"result": response}
+        )
         await self.callback_manager.run_callbacks(event)
         return response
 
@@ -253,7 +285,9 @@ class AIConfigRuntime(AIConfig):
             params = {}
 
         if prompt_name not in self.prompt_index:
-            raise IndexError(f"Prompt '{prompt_name}' not found in config, available prompts are:\n {list(self.prompt_index.keys())}")
+            raise IndexError(
+                f"Prompt '{prompt_name}' not found in config, available prompts are:\n {list(self.prompt_index.keys())}"
+            )
 
         prompt_data = self.prompt_index[prompt_name]
         model_name = self.get_model_name(prompt_data)
@@ -267,11 +301,12 @@ class AIConfigRuntime(AIConfig):
             self,
             options,
             params,
-            callback_manager=self.callback_manager,
-            **kwargs,
+            **kwargs,  # TODO: We should remove and make argument explicit
         )
 
-        event = CallbackEvent("on_run_complete", __name__, {"result": response})
+        event = CallbackEvent(
+            "on_run_complete", __name__, {"result": response}
+        )
         await self.callback_manager.run_callbacks(event)
         return response
 
@@ -317,7 +352,9 @@ class AIConfigRuntime(AIConfig):
 
         # Check if the provided prompt name is available in the list of prompts
         if prompt_name not in self.prompt_index:
-            raise IndexError(f"Prompt '{prompt_name}' not found in config, available prompts are:\n {list(self.prompt_index.keys())}")
+            raise IndexError(
+                f"Prompt '{prompt_name}' not found in config, available prompts are:\n {list(self.prompt_index.keys())}"
+            )
 
         # Retrieve model and respective provider
         prompt_data = self.prompt_index[prompt_name]
@@ -333,7 +370,6 @@ class AIConfigRuntime(AIConfig):
             self,
             parameters_list,
             options,
-            callback_manager=self.callback_manager,
             **kwargs,
         )
 
@@ -345,11 +381,25 @@ class AIConfigRuntime(AIConfig):
             parameters_dict_used = parameters_list[i]
 
             aiconfig_execute_results = aiconfig.get_prompt(prompt_name).outputs
-            prompt_data_resolved = await aiconfig.resolve(prompt_name, parameters_dict_used)
+            prompt_data_resolved = await aiconfig.resolve(
+                prompt_name, parameters_dict_used
+            )
 
-            batch_results_formatted.append(tuple([aiconfig_execute_results, prompt_data_resolved, parameters_dict_used]))
+            batch_results_formatted.append(
+                tuple(
+                    [
+                        aiconfig_execute_results,
+                        prompt_data_resolved,
+                        parameters_dict_used,
+                    ]
+                )
+            )
 
-        event = CallbackEvent("on_run_batch_complete", __name__, {"result": batch_results_formatted})
+        event = CallbackEvent(
+            "on_run_batch_complete",
+            __name__,
+            {"result": batch_results_formatted},
+        )
 
         await self.callback_manager.run_callbacks(event)
         return batch_results_formatted
@@ -361,10 +411,17 @@ class AIConfigRuntime(AIConfig):
         options: Optional[InferenceOptions] = None,
         **kwargs,
     ) -> str:
-        result: Any = await self.run(prompt_name, params, options=options, **kwargs)
+        result: Any = await self.run(
+            prompt_name, params, options=options, **kwargs
+        )
         return self.get_output_text(prompt_name, result[0])
 
-    def save(self, config_filepath: str | None = None, include_outputs: bool = True, mode: Literal["json", "yaml"] | None = None):
+    def save(
+        self,
+        config_filepath: str | None = None,
+        include_outputs: bool = True,
+        mode: Literal["json", "yaml"] | None = None,
+    ):
         """
         Save the AI Configuration to a file.
 
@@ -384,7 +441,9 @@ class AIConfigRuntime(AIConfig):
         if not include_outputs:
             exclude_options["prompts"] = {"__all__": {"outputs"}}
 
-        default_filepath = "aiconfig.yaml" if mode == "yaml" else "aiconfig.json"
+        default_filepath = (
+            "aiconfig.yaml" if mode == "yaml" else "aiconfig.json"
+        )
 
         if not config_filepath:
             config_filepath = self.file_path or default_filepath
@@ -412,14 +471,18 @@ class AIConfigRuntime(AIConfig):
                 )
             else:
                 # Save AIConfig as JSON to the file, with the schema specified
-                json_data["$schema"] = "https://json.schemastore.org/aiconfig-1.0"
+                json_data[
+                    "$schema"
+                ] = "https://json.schemastore.org/aiconfig-1.0"
                 json.dump(
                     json_data,
                     file,
                     indent=2,
                 )
 
-    def get_output_text(self, prompt: str | Prompt, output: Optional[dict] = None) -> str:
+    def get_output_text(
+        self, prompt: str | Prompt, output: Optional[dict] = None
+    ) -> str:
         """
         Get the string representing the output from a prompt (if any)
 
@@ -432,7 +495,9 @@ class AIConfigRuntime(AIConfig):
         """
         if isinstance(prompt, str):
             prompt = self.get_prompt(prompt)
-        model_parser = ModelParserRegistry.get_model_parser_for_prompt(prompt, self)
+        model_parser = ModelParserRegistry.get_model_parser_for_prompt(
+            prompt, self
+        )
         return model_parser.get_output_text(prompt, self, output)
 
     @staticmethod
@@ -457,10 +522,14 @@ class AIConfigRuntime(AIConfig):
             ModelParser: The model parser corresponding to the given identifier.
         """
         if model_id not in ModelParserRegistry.parser_ids():
-            raise IndexError(f"Model parser '{model_id}' not found in registry, available model parsers are:\n {ModelParserRegistry.parser_ids()}")
+            raise IndexError(
+                f"Model parser '{model_id}' not found in registry, available model parsers are:\n {ModelParserRegistry.parser_ids()}"
+            )
         return ModelParserRegistry.get_model_parser(model_id)
 
     def set_callback_manager(self, callback_manager: CallbackManager):
         if callback_manager is None:
-            raise ValueError("callback_manager cannot be None. Create a new CallbackManager with No callbacks instead.")
+            raise ValueError(
+                "callback_manager cannot be None. Create a new CallbackManager with No callbacks instead."
+            )
         self.callback_manager = callback_manager
