@@ -4,10 +4,16 @@ from typing import TYPE_CHECKING, Dict, List, Optional, Union
 
 import openai
 from aiconfig.callback import CallbackEvent
-from aiconfig.default_parsers.parameterized_model_parser import ParameterizedModelParser
+from aiconfig.default_parsers.parameterized_model_parser import (
+    ParameterizedModelParser,
+)
 from aiconfig.model_parser import InferenceOptions
 from aiconfig.util.config_utils import get_api_key_from_environment
-from aiconfig.util.params import resolve_prompt, resolve_prompt_string, resolve_system_prompt
+from aiconfig.util.params import (
+    resolve_prompt,
+    resolve_prompt_string,
+    resolve_system_prompt,
+)
 from openai.types.chat import ChatCompletionMessage
 
 from aiconfig.schema import (
@@ -29,6 +35,8 @@ if TYPE_CHECKING:
 class OpenAIInference(ParameterizedModelParser):
     def __init__(self):
         super().__init__()
+        # Will be set in the run method. This is to avoid having to set the api key in the constructor
+        self.client = None
 
     @abstractmethod
     def id(self) -> str:
@@ -72,7 +80,9 @@ class OpenAIInference(ParameterizedModelParser):
         conversation_data = {**data}
 
         if not "messages" in conversation_data:
-            raise ValueError("Data must have `messages` array to match openai api spec")
+            raise ValueError(
+                "Data must have `messages` array to match openai api spec"
+            )
 
         # Find first system prompt. Every prompt in the config will bet set to use this system prompt.
         system_prompt = None
@@ -83,9 +93,15 @@ class OpenAIInference(ParameterizedModelParser):
                 break
 
         # Get the global settings for the model
-        model_name = conversation_data["model"] if "model" in conversation_data else self.id()
+        model_name = (
+            conversation_data["model"]
+            if "model" in conversation_data
+            else self.id()
+        )
 
-        model_metadata = ai_config.get_model_metadata(conversation_data, model_name)
+        model_metadata = ai_config.get_model_metadata(
+            conversation_data, model_name
+        )
         # Remove messages array from model metadata. Handled separately
         model_metadata.settings.pop("messages", None)
 
@@ -107,7 +123,11 @@ class OpenAIInference(ParameterizedModelParser):
                         i += 1
                 new_prompt_name = f"{prompt_name}_{len(prompts) + 1}"
 
-                input = messsage["content"] if role == "user" else PromptInput(**messsage)
+                input = (
+                    messsage["content"]
+                    if role == "user"
+                    else PromptInput(**messsage)
+                )
 
                 assistant_output = []
                 if assistant_response is not None:
@@ -141,11 +161,18 @@ class OpenAIInference(ParameterizedModelParser):
         if prompts:
             prompts[len(prompts) - 1].name = prompt_name
 
-        event = CallbackEvent("on_serialize_complete", __name__, {"result": prompts})
+        event = CallbackEvent(
+            "on_serialize_complete", __name__, {"result": prompts}
+        )
         await ai_config.callback_manager.run_callbacks(event)
         return prompts
 
-    async def deserialize(self, prompt: Prompt, aiconfig: "AIConfigRuntime", params: Optional[Dict] = {}) -> Dict:
+    async def deserialize(
+        self,
+        prompt: Prompt,
+        aiconfig: "AIConfigRuntime",
+        params: Optional[Dict] = {},
+    ) -> Dict:
         """
         Defines how to parse a prompt in the .aiconfig for a particular model
         and constructs the completion params for that model.
@@ -156,11 +183,19 @@ class OpenAIInference(ParameterizedModelParser):
         Returns:
             dict: Model-specific completion parameters.
         """
-        await aiconfig.callback_manager.run_callbacks(CallbackEvent("on_deserialize_start", __name__, {"prompt": prompt, "params": params}))
+        await aiconfig.callback_manager.run_callbacks(
+            CallbackEvent(
+                "on_deserialize_start",
+                __name__,
+                {"prompt": prompt, "params": params},
+            )
+        )
         # Build Completion params
         model_settings = self.get_model_settings(prompt, aiconfig)
 
-        completion_params = refine_chat_completion_params(model_settings, aiconfig, prompt)
+        completion_params = refine_chat_completion_params(
+            model_settings, aiconfig, prompt
+        )
 
         # In the case thhat the messages array weren't saves as part of the model settings, build it here. Messages array is used for conversation history.
         if not completion_params.get("messages"):
@@ -172,12 +207,17 @@ class OpenAIInference(ParameterizedModelParser):
                 if isinstance(system_prompt, dict):
                     # If system prompt is an object, then it should have content and role attributes
                     system_prompt = system_prompt["content"]
-                resolved_system_prompt = resolve_system_prompt(prompt, system_prompt, params, aiconfig)
-                completion_params["messages"].append({"content": resolved_system_prompt, "role": "system"})
+                resolved_system_prompt = resolve_system_prompt(
+                    prompt, system_prompt, params, aiconfig
+                )
+                completion_params["messages"].append(
+                    {"content": resolved_system_prompt, "role": "system"}
+                )
 
             # Default to always use chat context
             if not hasattr(prompt.metadata, "remember_chat_context") or (
-                hasattr(prompt.metadata, "remember_chat_context") and prompt.metadata.remember_chat_context != False
+                hasattr(prompt.metadata, "remember_chat_context")
+                and prompt.metadata.remember_chat_context != False
             ):
                 # handle chat history. check previous prompts for the same model. if same model, add prompt and its output to completion data if it has a completed output
                 for i, previous_prompt in enumerate(aiconfig.prompts):
@@ -185,7 +225,9 @@ class OpenAIInference(ParameterizedModelParser):
                     if previous_prompt.name == prompt.name:
                         break
 
-                    if aiconfig.get_model_name(previous_prompt) == aiconfig.get_model_name(prompt):
+                    if aiconfig.get_model_name(
+                        previous_prompt
+                    ) == aiconfig.get_model_name(prompt):
                         # Add prompt and its output to completion data. Constructing this prompt will take into account available parameters.
                         add_prompt_as_message(
                             previous_prompt,
@@ -196,7 +238,9 @@ class OpenAIInference(ParameterizedModelParser):
         else:
             # If messages are already specified in the model settings, then just resolve each message with the given parameters and append the latest message
             for i in range(len(completion_params.get("messages"))):
-                completion_params["messages"][i]["content"] = resolve_prompt_string(
+                completion_params["messages"][i][
+                    "content"
+                ] = resolve_prompt_string(
                     prompt,
                     params,
                     aiconfig,
@@ -204,8 +248,16 @@ class OpenAIInference(ParameterizedModelParser):
                 )
 
         # Add in the latest prompt
-        add_prompt_as_message(prompt, aiconfig, completion_params["messages"], params)
-        await aiconfig.callback_manager.run_callbacks(CallbackEvent("on_deserialize_complete", __name__, {"output": completion_params}))
+        add_prompt_as_message(
+            prompt, aiconfig, completion_params["messages"], params
+        )
+        await aiconfig.callback_manager.run_callbacks(
+            CallbackEvent(
+                "on_deserialize_complete",
+                __name__,
+                {"output": completion_params},
+            )
+        )
         return completion_params
 
     async def run_inference(
@@ -230,12 +282,16 @@ class OpenAIInference(ParameterizedModelParser):
             CallbackEvent(
                 "on_run_start",
                 __name__,
-                {"prompt": prompt, "options": options, "parameters": parameters},
+                {
+                    "prompt": prompt,
+                    "options": options,
+                    "parameters": parameters,
+                },
             )
         )
 
-        if not openai.api_key:
-            openai.api_key = get_api_key_from_environment("OPENAI_API_KEY").unwrap()
+        if self.client is None:
+            self.initialize_openai_client()
 
         completion_data = await self.deserialize(prompt, aiconfig, parameters)
         # if stream enabled in runtime options and config, then stream. Otherwise don't stream.
@@ -249,19 +305,28 @@ class OpenAIInference(ParameterizedModelParser):
 
         completion_data["stream"] = stream
 
-        response = openai.chat.completions.create(**completion_data)
+        response = self.client.chat.completions.create(**completion_data)
         outputs = []
         if not stream:
             # # OpenAI>1.0.0 uses pydantic models for response
             response = response.model_dump(exclude_none=True)
 
-            response_without_choices = {key: copy.deepcopy(value) for key, value in response.items() if key != "choices"}
+            response_without_choices = {
+                key: copy.deepcopy(value)
+                for key, value in response.items()
+                if key != "choices"
+            }
             for i, choice in enumerate(response.get("choices")):
                 output_message = choice["message"]
                 output_data = build_output_data(output_message)
 
-                response_without_choices.update({"finish_reason": choice.get("finish_reason")})
-                metadata = {"raw_response": output_message, **response_without_choices}
+                response_without_choices.update(
+                    {"finish_reason": choice.get("finish_reason")}
+                )
+                metadata = {
+                    "raw_response": output_message,
+                    **response_without_choices,
+                }
                 if output_message.get("role", None) is not None:
                     metadata["role"] = output_message.get("role")
 
@@ -281,7 +346,11 @@ class OpenAIInference(ParameterizedModelParser):
             for chunk in response:
                 # OpenAI>1.0.0 uses pydantic models. Chunk is of type ChatCompletionChunk; type is not directly importable from openai Library, will require some diffing
                 chunk = chunk.model_dump(exclude_none=True)
-                chunk_without_choices = {key: copy.deepcopy(value) for key, value in chunk.items() if key != "choices"}
+                chunk_without_choices = {
+                    key: copy.deepcopy(value)
+                    for key, value in chunk.items()
+                    if key != "choices"
+                }
                 # streaming only returns one chunk, one choice at a time (before 1.0.0). The order in which the choices are returned is not guaranteed.
                 messages = multi_choice_message_reducer(messages, chunk)
 
@@ -291,7 +360,9 @@ class OpenAIInference(ParameterizedModelParser):
                     delta = choice.get("delta")
 
                     if options and options.stream_callback:
-                        options.stream_callback(delta, accumulated_message_for_choice, index)
+                        options.stream_callback(
+                            delta, accumulated_message_for_choice, index
+                        )
 
                     output = ExecuteResult(
                         **{
@@ -319,16 +390,35 @@ class OpenAIInference(ParameterizedModelParser):
         # rewrite or extend list of outputs?
         prompt.outputs = outputs
 
-        await aiconfig.callback_manager.run_callbacks(CallbackEvent("on_run_complete", __name__, {"result": prompt.outputs}))
+        await aiconfig.callback_manager.run_callbacks(
+            CallbackEvent(
+                "on_run_complete", __name__, {"result": prompt.outputs}
+            )
+        )
         return prompt.outputs
+    
+    def initialize_openai_client(self) -> None:
+        """
+        Initializes the client to be used with the OpenAI Module.
+        This method can be overriden to customize the client initialization.
+        """
 
-    def get_prompt_template(self, prompt: Prompt, aiconfig: "AIConfigRuntime") -> str:
+        openai_api_key = get_api_key_from_environment(
+            "OPENAI_API_KEY"
+        ).unwrap()
+        self.client = openai.Client(api_key=openai_api_key)   
+
+    def get_prompt_template(
+        self, prompt: Prompt, aiconfig: "AIConfigRuntime"
+    ) -> str:
         """
         Returns a template for a prompt.
         """
         if isinstance(prompt.input, str):
             return prompt.input
-        elif isinstance(prompt.input, PromptInput) and isinstance(prompt.input.data, str):
+        elif isinstance(prompt.input, PromptInput) and isinstance(
+            prompt.input.data, str
+        ):
             return prompt.input.data
         else:
             message = prompt.input
@@ -358,7 +448,10 @@ class OpenAIInference(ParameterizedModelParser):
             # Doing this to be backwards-compatible with old output format
             # where we used to save the ChatCompletionMessage in output.data
             if isinstance(output_data, ChatCompletionMessage):
-                if hasattr(output_data, "content") and output_data.content is not None:
+                if (
+                    hasattr(output_data, "content")
+                    and output_data.content is not None
+                ):
                     return output_data.content
                 elif output_data.function_call is not None:
                     return str(output_data.function_call)
@@ -391,7 +484,9 @@ def reduce(acc, delta):
     return acc
 
 
-def multi_choice_message_reducer(messages: Union[Dict[int, dict], None], chunk: dict) -> Dict[int, dict]:
+def multi_choice_message_reducer(
+    messages: Union[Dict[int, dict], None], chunk: dict
+) -> Dict[int, dict]:
     if messages is None:
         messages = {}
 
@@ -444,7 +539,9 @@ def refine_chat_completion_params(model_settings, aiconfig, prompt):
     return completion_data
 
 
-def add_prompt_as_message(prompt: Prompt, aiconfig: "AIConfigRuntime", messages: List, params=None):
+def add_prompt_as_message(
+    prompt: Prompt, aiconfig: "AIConfigRuntime", messages: List, params=None
+):
     """
     Converts a given prompt to a message and adds it to the specified messages list.
 
@@ -458,11 +555,17 @@ def add_prompt_as_message(prompt: Prompt, aiconfig: "AIConfigRuntime", messages:
         messages.append({"content": resolved_prompt, "role": "user"})
     else:
         # Assumes Prompt input will be in the format of ChatCompletionMessageParam (with content, role, function_name, and name attributes)
-        resolved_prompt = resolve_prompt_string(prompt, params, aiconfig, prompt.input.content)
+        resolved_prompt = resolve_prompt_string(
+            prompt, params, aiconfig, prompt.input.content
+        )
 
         prompt_input = prompt.input
         role = prompt_input.role if hasattr(prompt_input, "role") else "user"
-        fn_call = prompt_input.function_call if hasattr(prompt_input, "function_call") else None
+        fn_call = (
+            prompt_input.function_call
+            if hasattr(prompt_input, "function_call")
+            else None
+        )
         name = prompt_input.name if hasattr(prompt_input, "name") else None
 
         message_data = {"content": resolved_prompt, "role": role}
@@ -480,7 +583,10 @@ def add_prompt_as_message(prompt: Prompt, aiconfig: "AIConfigRuntime", messages:
         if output.output_type == "execute_result":
             assert isinstance(output, ExecuteResult)
             output_data = output.data
-            role = output.metadata.get("role", None) or ("raw_response" in output.metadata and output.metadata["raw_response"].get("role", None))
+            role = output.metadata.get("role", None) or (
+                "raw_response" in output.metadata
+                and output.metadata["raw_response"].get("role", None)
+            )
 
             if role == "assistant":
                 output_message = {}
@@ -492,8 +598,12 @@ def add_prompt_as_message(prompt: Prompt, aiconfig: "AIConfigRuntime", messages:
                     if isinstance(output_data.value, str):
                         content = output_data.value
                     elif output_data.kind == "tool_calls":
-                        assert isinstance(output_data, OutputDataWithToolCallsValue)
-                        function_call = output_data.value[len(output_data.value) - 1].function
+                        assert isinstance(
+                            output_data, OutputDataWithToolCallsValue
+                        )
+                        function_call = output_data.value[
+                            len(output_data.value) - 1
+                        ].function
 
                 output_message["content"] = content
                 output_message["role"] = role
@@ -504,7 +614,10 @@ def add_prompt_as_message(prompt: Prompt, aiconfig: "AIConfigRuntime", messages:
                 if function_call is not None:
                     output_message["function_call"] = function_call
 
-                name = output.metadata.get("name", None) or ("raw_response" in output.metadata and output.metadata["raw_response"].get("name", None))
+                name = output.metadata.get("name", None) or (
+                    "raw_response" in output.metadata
+                    and output.metadata["raw_response"].get("name", None)
+                )
                 if name is not None:
                     output_message["name"] = name
 
@@ -523,7 +636,9 @@ def is_prompt_template(prompt: Prompt):
     """
     Check if a prompt's input is a valid string.
     """
-    return isinstance(prompt.input, str) or (hasattr(prompt.input, "data") and isinstance(prompt.input.data, str))
+    return isinstance(prompt.input, str) or (
+        hasattr(prompt.input, "data") and isinstance(prompt.input.data, str)
+    )
 
 
 def build_output_data(
