@@ -2,19 +2,22 @@ import asyncio
 import copy
 import ctypes
 import json
-import copy
 import logging
 import threading
 import time
-import webbrowser
 import uuid
+import webbrowser
 from typing import Any, Dict, Type, Union
 
 import lastmile_utils.lib.core.api as core_utils
 import result
 from aiconfig.Config import AIConfigRuntime
-from aiconfig.editor.server.queue_iterator import STOP_STREAMING_SIGNAL, QueueIterator
+from aiconfig.editor.server.queue_iterator import (
+    STOP_STREAMING_SIGNAL,
+    QueueIterator,
+)
 from aiconfig.editor.server.server_utils import (
+    AIConfigRC,
     EditServerConfig,
     FlaskResponse,
     HttpResponseWithAIConfig,
@@ -57,35 +60,56 @@ app = Flask(__name__, static_url_path="")
 CORS(app, resources={r"/api/*": {"origins": "*"}})
 
 
-def run_backend_server(edit_config: EditServerConfig) -> Result[str, str]:
+def run_backend_server(
+    edit_config: EditServerConfig, aiconfigrc_path: str
+) -> Result[str, str]:
     LOGGER.setLevel(edit_config.log_level)
     LOGGER.info("Edit config: %s", edit_config.model_dump_json())
-    LOGGER.info(f"Starting server on http://localhost:{edit_config.server_port}")
+    LOGGER.info(
+        f"Starting server on http://localhost:{edit_config.server_port}"
+    )
     try:
-        LOGGER.info(f"Opening browser at http://localhost:{edit_config.server_port}")
+        LOGGER.info(
+            f"Opening browser at http://localhost:{edit_config.server_port}"
+        )
         webbrowser.open(f"http://localhost:{edit_config.server_port}")
     except Exception as e:
-        LOGGER.warning(f"Failed to open browser: {e}. Please open http://localhost:{port} manually.")
+        LOGGER.warning(
+            f"Failed to open browser: {e}. Please open http://localhost:{port} manually."
+        )
 
     app.server_state = ServerState()  # type: ignore
-    res_server_state_init = init_server_state(app, edit_config)
+    res_server_state_init = init_server_state(
+        app, edit_config, aiconfigrc_path
+    )
     match res_server_state_init:
         case Ok(_):
             LOGGER.info("Initialized server state")
-            debug = edit_config.server_mode in [ServerMode.DEBUG_BACKEND, ServerMode.DEBUG_SERVERS]
+            debug = edit_config.server_mode in [
+                ServerMode.DEBUG_BACKEND,
+                ServerMode.DEBUG_SERVERS,
+            ]
             LOGGER.info(f"Running in {edit_config.server_mode} mode")
-            app.run(port=edit_config.server_port, debug=debug, use_reloader=debug)
+            app.run(
+                port=edit_config.server_port, debug=debug, use_reloader=debug
+            )
             return Ok("Done")
         case Err(e):
             LOGGER.error(f"Failed to initialize server state: {e}")
             return Err(f"Failed to initialize server state: {e}")
 
 
-def _validated_request_path(request_json: core_utils.JSONObject, allow_create: bool = False) -> Result[ValidatedPath, str]:
+def _validated_request_path(
+    request_json: core_utils.JSONObject, allow_create: bool = False
+) -> Result[ValidatedPath, str]:
     if "path" not in request_json or not isinstance(request_json["path"], str):
-        return Err("Request JSON must contain a 'path' field with a string value.")
+        return Err(
+            "Request JSON must contain a 'path' field with a string value."
+        )
     else:
-        return get_validated_path(request_json["path"], allow_create=allow_create)
+        return get_validated_path(
+            request_json["path"], allow_create=allow_create
+        )
 
 
 @app.route("/")
@@ -115,7 +139,11 @@ def load_model_parser_module() -> FlaskResponse:
         case Ok(resp):
             return resp.to_flask_format()
         case Err(e):
-            return HttpResponseWithAIConfig(message=f"Failed to load model parser module: {e}", code=400, aiconfig=None).to_flask_format()
+            return HttpResponseWithAIConfig(
+                message=f"Failed to load model parser module: {e}",
+                code=400,
+                aiconfig=None,
+            ).to_flask_format()
 
 
 @app.route("/api/load", methods=["POST"])
@@ -124,25 +152,40 @@ def load() -> FlaskResponse:
     request_json = request.get_json()
     if not request_json.keys() <= {"path"}:
         return HttpResponseWithAIConfig(
-            message="Request JSON must contain a 'path' field with a string value, or no arguments.", code=400, aiconfig=None
+            message="Request JSON must contain a 'path' field with a string value, or no arguments.",
+            code=400,
+            aiconfig=None,
         ).to_flask_format()
     path: str | None = request_json.get("path", None)
     if path is None:
         aiconfig = state.aiconfig
         if aiconfig is None:
-            return HttpResponseWithAIConfig(message="No AIConfig loaded", code=400, aiconfig=None).to_flask_format()
+            return HttpResponseWithAIConfig(
+                message="No AIConfig loaded", code=400, aiconfig=None
+            ).to_flask_format()
         else:
-            return HttpResponseWithAIConfig(message="AIConfig already loaded. Here it is!", aiconfig=aiconfig).to_flask_format()
+            return HttpResponseWithAIConfig(
+                message="AIConfig already loaded. Here it is!",
+                aiconfig=aiconfig,
+            ).to_flask_format()
     else:
         res_path_val = get_validated_path(path)
         res_aiconfig = res_path_val.and_then(safe_load_from_disk)
         match res_aiconfig:
             case Ok(aiconfig):
-                LOGGER.warning(f"Loaded AIConfig from {res_path_val}. This may have overwritten in-memory changes.")
+                LOGGER.warning(
+                    f"Loaded AIConfig from {res_path_val}. This may have overwritten in-memory changes."
+                )
                 state.aiconfig = aiconfig
-                return HttpResponseWithAIConfig(message="Loaded", aiconfig=aiconfig).to_flask_format()
+                return HttpResponseWithAIConfig(
+                    message="Loaded", aiconfig=aiconfig
+                ).to_flask_format()
             case Err(e):
-                return HttpResponseWithAIConfig(message=f"Failed to load AIConfig: {res_path_val}, {e}", code=400, aiconfig=None).to_flask_format()
+                return HttpResponseWithAIConfig(
+                    message=f"Failed to load AIConfig: {res_path_val}, {e}",
+                    code=400,
+                    aiconfig=None,
+                ).to_flask_format()
 
 
 @app.route("/api/save", methods=["POST"])
@@ -154,32 +197,52 @@ def save() -> FlaskResponse:
 
     if path is None:
         if aiconfig is None:
-            return HttpResponseWithAIConfig(message="No AIConfig loaded", code=400, aiconfig=None).to_flask_format()
+            return HttpResponseWithAIConfig(
+                message="No AIConfig loaded", code=400, aiconfig=None
+            ).to_flask_format()
         else:
-            LOGGER.info(f"No path provided, saving to original path, {aiconfig.file_path}")
+            LOGGER.info(
+                f"No path provided, saving to original path, {aiconfig.file_path}"
+            )
             path = aiconfig.file_path
 
     res_path_val = get_validated_path(path, allow_create=True)
     match res_path_val:
         case Ok(path_ok):
             _op = make_op_run_method(MethodName("save"))
-            op_args: Result[OpArgs, str] = result.Ok(OpArgs({"config_filepath": path_ok}))
-            return run_aiconfig_operation_with_op_args(aiconfig, "save", _op, op_args)
+            op_args: Result[OpArgs, str] = result.Ok(
+                OpArgs({"config_filepath": path_ok})
+            )
+            return run_aiconfig_operation_with_op_args(
+                aiconfig, "save", _op, op_args
+            )
 
         case Err(e):
-            return HttpResponseWithAIConfig(message=f"Failed to save AIConfig: {e}", code=400, aiconfig=None).to_flask_format()
+            return HttpResponseWithAIConfig(
+                message=f"Failed to save AIConfig: {e}",
+                code=400,
+                aiconfig=None,
+            ).to_flask_format()
 
 
 @app.route("/api/create", methods=["POST"])
 def create() -> FlaskResponse:
     state = get_server_state(app)
-    aiconfig = safe_run_aiconfig_static_method(MethodName("create"), OpArgs({}), AIConfigRuntime)
+    aiconfig = safe_run_aiconfig_static_method(
+        MethodName("create"), OpArgs({}), AIConfigRuntime
+    )
     match aiconfig:
         case Ok(aiconfig_ok):
             state.aiconfig = aiconfig_ok
-            return HttpResponseWithAIConfig(message="Created new AIConfig", aiconfig=aiconfig_ok).to_flask_format()
+            return HttpResponseWithAIConfig(
+                message="Created new AIConfig", aiconfig=aiconfig_ok
+            ).to_flask_format()
         case Err(e):
-            return HttpResponseWithAIConfig(message=f"Failed to create AIConfig: {e}", code=400, aiconfig=None).to_flask_format()
+            return HttpResponseWithAIConfig(
+                message=f"Failed to create AIConfig: {e}",
+                code=400,
+                aiconfig=None,
+            ).to_flask_format()
 
 
 @app.route("/api/run", methods=["POST"])
@@ -217,7 +280,9 @@ def run() -> FlaskResponse:
     # Define stream callback and queue object for streaming results
     output_text_queue = QueueIterator()
 
-    def update_output_queue(data: str, _accumulated_data: str, _index: int) -> None:
+    def update_output_queue(
+        data: str, _accumulated_data: str, _index: int
+    ) -> None:
         should_end_stream: bool = data == STOP_STREAMING_SIGNAL
         output_text_queue.put(data, should_end_stream)
 
@@ -243,11 +308,25 @@ def run() -> FlaskResponse:
             output_text_queue.put(STOP_STREAMING_SIGNAL)  # type: ignore
 
         def create_error_payload(message: str, code: int):
-            aiconfig_json = aiconfig_deep_copy.model_dump(exclude=EXCLUDE_OPTIONS) if aiconfig_deep_copy is not None else None
-            return json.dumps({"error": {"message": message, "code": code, "data": aiconfig_json}})
+            aiconfig_json = (
+                aiconfig_deep_copy.model_dump(exclude=EXCLUDE_OPTIONS)
+                if aiconfig_deep_copy is not None
+                else None
+            )
+            return json.dumps(
+                {
+                    "error": {
+                        "message": message,
+                        "code": code,
+                        "data": aiconfig_json,
+                    }
+                }
+            )
 
         def create_cancellation_payload():
-            return create_error_payload(message="The task was cancelled.", code=499)
+            return create_error_payload(
+                message="The task was cancelled.", code=499
+            )
 
         def handle_cancellation():
             yield "["
@@ -271,7 +350,9 @@ def run() -> FlaskResponse:
                 # Nothing to do
                 return
 
-            response = ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(thread_id), ctypes.py_object(SystemExit))
+            response = ctypes.pythonapi.PyThreadState_SetAsyncExc(
+                ctypes.c_long(thread_id), ctypes.py_object(SystemExit)
+            )
 
             if response == 0:
                 print(f"Invalid thread id {thread_id}")
@@ -284,11 +365,6 @@ def run() -> FlaskResponse:
         t = threading.Thread(target=run_async_config_in_thread)
         t.start()
 
-        # Create a deep copy of the state aiconfig so we can yield an AIConfig
-        # with streaming partial outputs in the meantime. This probably isn't
-        # necessary, but just getting unblocked for now
-        displaying_config = copy.deepcopy(aiconfig)
-
         # If model supports streaming, need to wait until streamer has at
         # least 1 item to display. If model does not support streaming,
         # need to wait until the aiconfig.run() thread is complete
@@ -303,7 +379,9 @@ def run() -> FlaskResponse:
             # we can fix later
             time.sleep(0.1)
             wait_time_in_seconds += SLEEP_DELAY_SECONDS
-            print(f"Output queue is currently empty. Waiting for {wait_time_in_seconds:.1f}s...")
+            print(
+                f"Output queue is currently empty. Waiting for {wait_time_in_seconds:.1f}s..."
+            )
 
         # Yield in flask is weird and you either need to send responses as a
         # string, or artificially wrap them around "[" and "]"
@@ -316,7 +394,9 @@ def run() -> FlaskResponse:
                     return
 
                 if isinstance(text, Exception):
-                    yield from create_error_payload(message=f"Exception: {text}", code=500)
+                    yield from create_error_payload(
+                        message=f"Exception: {text}", code=500
+                    )
                     return
                 elif isinstance(text, str):
                     accumulated_output_text += text
@@ -338,11 +418,10 @@ def run() -> FlaskResponse:
                         "metadata": {},
                     }  # type: ignore
                 )
-
-                displaying_config.add_output(prompt_name, accumulated_output, overwrite=True)
-                aiconfig_json = displaying_config.model_dump(exclude=EXCLUDE_OPTIONS)
                 yield "["
-                yield json.dumps({"aiconfig": aiconfig_json})
+                yield json.dumps(
+                    {"output_chunk": accumulated_output.to_json()}
+                )
                 yield "]"
 
         # Ensure that the run process is complete to yield final output
@@ -354,13 +433,23 @@ def run() -> FlaskResponse:
         else:
             state.events.pop(cancellation_token_id, None)
 
-            aiconfig_json = aiconfig.model_dump(exclude=EXCLUDE_OPTIONS) if aiconfig is not None else None
+            aiconfig_json = (
+                aiconfig.model_dump(exclude=EXCLUDE_OPTIONS)
+                if aiconfig is not None
+                else None
+            )
             yield "["
-            yield json.dumps({"aiconfig_complete": aiconfig_json})
+            yield json.dumps({"aiconfig_chunk": aiconfig_json})
             yield "]"
 
+        yield "["
+        yield json.dumps({"stop_streaming": None})
+        yield "]"
+
     try:
-        LOGGER.info(f"Running `aiconfig.run()` command with request: {request_json}")
+        LOGGER.info(
+            f"Running `aiconfig.run()` command with request: {request_json}"
+        )
         # Note; We run the streaming API even for non-streaming runs so that
         # we can unify the way we process data on the client
         # Streaming based on
@@ -384,7 +473,9 @@ def cancel() -> FlaskResponse:
     state = get_server_state(app)
     request_json = request.get_json()
 
-    cancellation_token_id: str | None = request_json.get("cancellation_token_id")
+    cancellation_token_id: str | None = request_json.get(
+        "cancellation_token_id"
+    )
     if cancellation_token_id is not None:
         event = state.events.get(cancellation_token_id)
         if event is not None:
@@ -392,7 +483,9 @@ def cancel() -> FlaskResponse:
             # Remove the event from the events dict
             state.events.pop(cancellation_token_id)
 
-            return FlaskResponse(({"cancellation_token_id": cancellation_token_id}, 200))
+            return FlaskResponse(
+                ({"cancellation_token_id": cancellation_token_id}, 200)
+            )
         else:
             # Return a 422 Unprocessable Entity
             return FlaskResponse(
@@ -419,27 +512,38 @@ def cancel() -> FlaskResponse:
 @app.route("/api/add_prompt", methods=["POST"])
 def add_prompt() -> FlaskResponse:
     method_name = MethodName("add_prompt")
-    signature: dict[str, Type[Any]] = {"prompt_name": str, "prompt_data": Prompt, "index": int}
+    signature: dict[str, Type[Any]] = {
+        "prompt_name": str,
+        "prompt_data": Prompt,
+        "index": int,
+    }
 
     state = get_server_state(app)
     aiconfig = state.aiconfig
     request_json = request.get_json()
 
     operation = make_op_run_method(method_name)
-    return run_aiconfig_operation_with_request_json(aiconfig, request_json, f"method_{method_name}", operation, signature)
+    return run_aiconfig_operation_with_request_json(
+        aiconfig, request_json, f"method_{method_name}", operation, signature
+    )
 
 
 @app.route("/api/update_prompt", methods=["POST"])
 def update_prompt() -> FlaskResponse:
     method_name = MethodName("update_prompt")
-    signature: dict[str, Type[Any]] = {"prompt_name": str, "prompt_data": Prompt}
+    signature: dict[str, Type[Any]] = {
+        "prompt_name": str,
+        "prompt_data": Prompt,
+    }
 
     state = get_server_state(app)
     aiconfig = state.aiconfig
     request_json = request.get_json()
 
     operation = make_op_run_method(method_name)
-    return run_aiconfig_operation_with_request_json(aiconfig, request_json, f"method_{method_name}", operation, signature)
+    return run_aiconfig_operation_with_request_json(
+        aiconfig, request_json, f"method_{method_name}", operation, signature
+    )
 
 
 @app.route("/api/delete_prompt", methods=["POST"])
@@ -452,7 +556,9 @@ def delete_prompt() -> FlaskResponse:
     request_json = request.get_json()
 
     operation = make_op_run_method(method_name)
-    return run_aiconfig_operation_with_request_json(aiconfig, request_json, f"method_{method_name}", operation, signature)
+    return run_aiconfig_operation_with_request_json(
+        aiconfig, request_json, f"method_{method_name}", operation, signature
+    )
 
 
 @app.route("/api/update_model", methods=["POST"])
@@ -466,8 +572,18 @@ def update_model() -> FlaskResponse:
     prompt_name: str | None = request_json.get("prompt_name")
 
     operation = make_op_run_method(MethodName("update_model"))
-    operation_args: Result[OpArgs, str] = result.Ok(OpArgs({"model_name": model_name, "settings": settings, "prompt_name": prompt_name}))
-    return run_aiconfig_operation_with_op_args(aiconfig, "update_model", operation, operation_args)
+    operation_args: Result[OpArgs, str] = result.Ok(
+        OpArgs(
+            {
+                "model_name": model_name,
+                "settings": settings,
+                "prompt_name": prompt_name,
+            }
+        )
+    )
+    return run_aiconfig_operation_with_op_args(
+        aiconfig, "update_model", operation, operation_args
+    )
 
 
 @app.route("/api/set_parameter", methods=["POST"])
@@ -477,14 +593,24 @@ def set_parameter() -> FlaskResponse:
     request_json = request.get_json()
 
     parameter_name: str | None = request_json.get("parameter_name")
-    parameter_value: Union[str, Dict[str, Any]] | None = request_json.get("parameter_value")
+    parameter_value: Union[str, Dict[str, Any]] | None = request_json.get(
+        "parameter_value"
+    )
     prompt_name: str | None = request_json.get("prompt_name")
 
     operation = make_op_run_method(MethodName("set_parameter"))
     operation_args: Result[OpArgs, str] = result.Ok(
-        OpArgs({"parameter_name": parameter_name, "parameter_value": parameter_value, "prompt_name": prompt_name})
+        OpArgs(
+            {
+                "parameter_name": parameter_name,
+                "parameter_value": parameter_value,
+                "prompt_name": prompt_name,
+            }
+        )
+    )
+    return run_aiconfig_operation_with_op_args(
+        aiconfig, "set_parameter", operation, operation_args
     )
-    return run_aiconfig_operation_with_op_args(aiconfig, "set_parameter", operation, operation_args)
 
 
 @app.route("/api/set_parameters", methods=["POST"])
@@ -497,8 +623,12 @@ def set_parameters() -> FlaskResponse:
     prompt_name: str | None = request_json.get("prompt_name")
 
     operation = make_op_run_method(MethodName("set_parameters"))
-    operation_args: Result[OpArgs, str] = result.Ok(OpArgs({"parameters": parameters, "prompt_name": prompt_name}))
-    return run_aiconfig_operation_with_op_args(aiconfig, "set_parameters", operation, operation_args)
+    operation_args: Result[OpArgs, str] = result.Ok(
+        OpArgs({"parameters": parameters, "prompt_name": prompt_name})
+    )
+    return run_aiconfig_operation_with_op_args(
+        aiconfig, "set_parameters", operation, operation_args
+    )
 
 
 @app.route("/api/delete_parameter", methods=["POST"])
@@ -511,8 +641,12 @@ def delete_parameter() -> FlaskResponse:
     prompt_name: str | None = request_json.get("prompt_name")
 
     operation = make_op_run_method(MethodName("delete_parameter"))
-    operation_args: Result[OpArgs, str] = result.Ok(OpArgs({"parameter_name": parameter_name, "prompt_name": prompt_name}))
-    return run_aiconfig_operation_with_op_args(aiconfig, "delete_parameter", operation, operation_args)
+    operation_args: Result[OpArgs, str] = result.Ok(
+        OpArgs({"parameter_name": parameter_name, "prompt_name": prompt_name})
+    )
+    return run_aiconfig_operation_with_op_args(
+        aiconfig, "delete_parameter", operation, operation_args
+    )
 
 
 @app.route("/api/set_name", methods=["POST"])
@@ -525,7 +659,9 @@ def set_name() -> FlaskResponse:
 
     operation = make_op_run_method(MethodName("set_name"))
     operation_args: Result[OpArgs, str] = result.Ok(OpArgs({"name": name}))
-    return run_aiconfig_operation_with_op_args(aiconfig, "set_name", operation, operation_args)
+    return run_aiconfig_operation_with_op_args(
+        aiconfig, "set_name", operation, operation_args
+    )
 
 
 @app.route("/api/set_description", methods=["POST"])
@@ -537,8 +673,12 @@ def set_description() -> FlaskResponse:
     description: str | None = request_json.get("description")
 
     operation = make_op_run_method(MethodName("set_description"))
-    operation_args: Result[OpArgs, str] = result.Ok(OpArgs({"description": description}))
-    return run_aiconfig_operation_with_op_args(aiconfig, "set_description", operation, operation_args)
+    operation_args: Result[OpArgs, str] = result.Ok(
+        OpArgs({"description": description})
+    )
+    return run_aiconfig_operation_with_op_args(
+        aiconfig, "set_description", operation, operation_args
+    )
 
 
 @app.route("/api/clear_outputs", methods=["POST"])
@@ -558,7 +698,9 @@ def clear_outputs() -> FlaskResponse:
             aiconfig=None,
         ).to_flask_format()
 
-    def _op(aiconfig_runtime: AIConfigRuntime, _op_args: OpArgs) -> Result[None, str]:
+    def _op(
+        aiconfig_runtime: AIConfigRuntime, _op_args: OpArgs
+    ) -> Result[None, str]:
         for prompt in aiconfig_runtime.prompts:
             prompt_name = prompt.name
             # fn name `delete_output`` is misleading. TODO: Rename to `delete_outputs`` in AIConfig API
@@ -566,4 +708,41 @@ def clear_outputs() -> FlaskResponse:
         return Ok(None)
 
     signature: dict[str, Type[Any]] = {}
-    return run_aiconfig_operation_with_request_json(aiconfig, request_json, f"method_", _op, signature)
+    return run_aiconfig_operation_with_request_json(
+        aiconfig, request_json, f"method_", _op, signature
+    )
+
+
+@app.route("/api/get_aiconfigrc", methods=["GET"])
+def get_aiconfigrc() -> FlaskResponse:
+    state = get_server_state(app)
+
+    yaml_mapping: Result[AIConfigRC, str] = core_utils.read_text_file(
+        state.aiconfigrc_path
+    ).and_then(AIConfigRC.from_yaml)
+    match yaml_mapping:
+        case Ok(yaml_mapping_ok):
+            return FlaskResponse((yaml_mapping_ok.model_dump(), 200))
+        case Err(e):
+            return FlaskResponse(
+                ({"message": f"Failed to load aiconfigrc: {e}"}, 400)
+            )
+
+
+@app.route("/api/set_aiconfigrc", methods=["POST"])
+def set_aiconfigrc() -> FlaskResponse:
+    get_server_state(app)
+    request.get_json()
+    # TODO:
+    # We might not need to implement this at all.
+    #
+    # If so:
+    # Assuming request_json["aiconfigrc"] is a yaml-formatted string
+    # (possibly with comments)
+    # Note that the file might already exist and have contents.
+    #
+    # here's how to write it to a file:
+    # from ruamel.yaml import YAML
+    # yaml = YAML()
+    # with open(state.aiconfigrc_path, "w") as f:
+    #     yaml.dump(request_json["aiconfigrc"], f)
